using System.IO.Enumeration;
using System.Security.Cryptography;

if (args.Length < 1)
{
    Console.WriteLine("Usage: Md5SumJ <dir-tree-to-checksum> [parallel:default same to vCpu]");
    return -1;
}

var dirToCheckSum = args[0];

var threads = Environment.ProcessorCount;
if (args.Length > 1 && ushort.TryParse(args[1], out var t)) threads = t;

var files = new FileSystemEnumerable<string>(dirToCheckSum, (ref FileSystemEntry p) => p.ToFullPath(),
        new EnumerationOptions { RecurseSubdirectories = true, })
{ ShouldIncludePredicate = (ref FileSystemEntry p) => p.IsDirectory == false };

Parallel.ForEach(files,
    new ParallelOptions { MaxDegreeOfParallelism = threads },
    f =>
    {
        try
        {
            Console.WriteLine($"{MakeMd5FromFilePath(f)}   {f} | by thread#{Environment.CurrentManagedThreadId}");
        }
        catch (Exception e)
        {
            Console.WriteLine($"Err: {e.Message}");
        }
    });

return 0;

static string MakeMd5FromFilePath(string filePath)
{
    using var fs = File.OpenRead(filePath);
    var md5 = MD5.HashData(fs);
    return Convert.ToHexString(md5);
}
